@@ -222,23 +222,49 @@ class DBStorage(BaseStorage):
         # check NIP05 verification, if enabled
         await self.verifier.verify(conn, event)
 
-        if event.is_replaceable:
+        if event.is_replaceable or event.is_paramaterized_replaceable:
             # check for older event from same pubkey
-            result = await conn.execute(
-                    sa.select(
-                        self.EventTable.c.id, self.EventTable.c.created_at
+            query = sa.select(
+                        self.EventTable.c.id, self.EventTable.c.created_at, self.EventTable.c.tags
                     ).where(
                         (self.EventTable.c.pubkey == bytes.fromhex(event.pubkey)) & 
                         (self.EventTable.c.kind == event.kind) & 
                         (self.EventTable.c.created_at < event.created_at)
                     )
-            )
-            row = result.first()
-            if row:
-                old_id = row[0]
-                old_ts = row[1]
-                self.log.info("Replacing event %s from %s@%s with %s", old_id, event.pubkey, old_ts, event.id)
-                await conn.execute(self.EventTable.delete().where(self.EventTable.c.id == old_id))
+            result = await conn.execute(query)
+
+            delete_id = None
+            if event.is_paramaterized_replaceable:
+                # according to nip-33, an event with a matching "d" tag will be replaced
+                # empty tags include [], [["d"]], and [["d", ""]]
+                d_tag = ''
+                for tag in event.tags:
+                    if tag[0] == 'd':
+                        if len(tag) > 1:
+                            d_tag = tag[1]
+                        break
+                for old_id, created_at, tags in result:
+                    found_tag = [tag for tag in tags if tag[0] == 'd']
+                    if not d_tag:
+                        if not found_tag or len(found_tag[0]) == 1 or found_tag[0][1] == '':
+                            delete_id = old_id
+                            old_ts = created_at
+                            break
+                    else:
+                        tag = found_tag[0]
+                        if len(tag) > 1 and tag[1] == d_tag:
+                            delete_id = old_id
+                            old_ts = created_at
+                            break
+
+            else:
+                row = result.first()
+                if row:
+                    delete_id = row[0]
+                    old_ts = row[1]
+            if delete_id:
+                self.log.info("Replacing event %s from %s@%s with %s", delete_id, event.pubkey, old_ts, event.id)
+                await conn.execute(self.EventTable.delete().where(self.EventTable.c.id == delete_id))
         return True
 
     async def process_tags(self, conn, event):
@@ -252,7 +278,7 @@ class DBStorage(BaseStorage):
                 if tag[0] in ('delegation', 'expiration'):
                     tags.add((tag[0], tag[1]))
                 elif len(tag[0]) == 1:
-                    tags.add((tag[0], tag[1]))
+                    tags.add((tag[0], tag[1] if len(tag) > 1 else ''))
             if tags:
                 result = await conn.execute(self.tag_insert_query,
                     [{'id': event.id_bytes, 'name': tag[0], 'value': tag[1]} for tag in tags]
@@ -442,6 +468,7 @@ class Subscription:
         self.default_limit = default_limit
         self.is_postgres = is_postgres
         self.log = log
+        self.long_query_threshold = 500
 
     def prepare(self):
         try:
@@ -475,8 +502,9 @@ class Subscription:
 
             duration = t()
             self.log.info('%s/%s query – events:%s duration:%dms', self.client_id, self.sub_id, count, duration)
-            if duration > 500:
-                self.log.warning("%s/%s Long query: '%s' took %dms", self.client_id, self.sub_id, rapidjson.dumps(self.filters), duration)
+            if duration > self.long_query_threshold:
+                logging.getLogger("nostr_relay.long-queries").warning("%s/%s Long query: '%s' took %dms", self.client_id, self.sub_id, rapidjson.dumps(self.filters), duration)
+
         except Exception:
             self.log.exception("subscription")
 
@@ -495,6 +523,8 @@ class Subscription:
 
     def check_event(self, event, filters):
         for filter_obj in filters:
+            if not filter_obj:
+                continue
             matched = set()
             for key, value in filter_obj.items():
                 if key == 'ids':
@@ -514,6 +544,11 @@ class Subscription:
                     for tag in event.tags:
                         if tag[0] == key[1]:
                             matched.add(tag[1] in value)
+                elif key == 'limit':
+                    # limit is irrelevant for broadcasts
+                    continue
+                else:
+                    matched.add(False)
             if all(matched):
                 return True
         return False
@@ -524,7 +559,7 @@ class Subscription:
                 if not isinstance(value, list):
                     value = [value]
                 ids = set(value)
-                if ids and len(ids) < 40: # pathological queries?
+                if ids:
                     exact = []
                     for eid in ids:
                         eid = validate_id(eid)
@@ -546,7 +581,7 @@ class Subscription:
                     # invalid query
                     raise ValueError("ids")
             elif key == 'authors' and isinstance(value, list):
-                if value and len(value) < 40:
+                if value:
                     exact = set()
                     hexexact = set()
                     for pubkey in value:
@@ -582,8 +617,8 @@ class Subscription:
                     subwhere.append('created_at < %d' % int(value))
                 else:
                     raise ValueError("until")
-            elif key == 'limit' and value:
-                filter_obj['limit'] = max(min(int(value or 0), self.default_limit), 0)
+            elif key == 'limit' and isinstance(value, int):
+                filter_obj['limit'] = min(max(value, 0), self.default_limit)
             elif key[0] == '#' and len(key) == 2 and value:
                 pstr = []
                 for val in set(value):
