@@ -5,13 +5,14 @@ import falcon
 import rapidjson
 from time import time
 from falcon import media
-from websockets.exceptions import ConnectionClosedError
+from websockets.exceptions import ConnectionClosedError, ConnectionClosedOK
 
 import falcon.asgi
 
 from .rate_limiter import get_rate_limiter
 from . import __version__
 from .config import Config
+from .util import timeout
 from .errors import AuthenticationError, StorageError
 
 
@@ -56,7 +57,7 @@ class Client:
                 await ws.send_text(message)
                 self.log.debug("SENT: %s", message)
                 self.sent += len(message)
-            except (falcon.WebSocketDisconnected, ConnectionClosedError):
+            except (falcon.WebSocketDisconnected, ConnectionClosedError, ConnectionClosedOK):
                 break
             except asyncio.CancelledError:
                 break
@@ -70,11 +71,12 @@ class Client:
 
         if storage.authenticator.is_enabled:
             challenge = storage.authenticator.get_challenge(remote_addr)
+            self.log.debug("Sent challenge %s to %s", challenge, client_id)
             await ws.send_media(["AUTH", challenge])
 
         while ws.ready:
             try:
-                async with asyncio.timeout(self.timeout):
+                async with timeout(self.timeout):
                     message = await ws.receive_media()
 
                 if not self.validate_message(message):
@@ -159,7 +161,7 @@ class NostrAPI(BaseResource):
 
     async def on_get(self, req: falcon.Request, resp: falcon.Response):
         if req.accept == 'application/nostr+json':
-            supported_nips = [1, 2, 5, 9, 11, 12, 15, 20, 26, 40]
+            supported_nips = [1, 2, 5, 9, 11, 12, 15, 20, 26, 33, 40]
             if Config.authentication.get('enabled'):
                 supported_nips.append(42)
             metadata = {
@@ -189,7 +191,7 @@ class NostrAPI(BaseResource):
                 return
 
         try:
-            if self.rate_limiter and self.rate_limiter.is_limited(req.remote_addr, 'ACCEPT'):
+            if self.rate_limiter and self.rate_limiter.is_limited(req.remote_addr, ['ACCEPT']):
                 await ws.close(code=1013)
                 self.log.warning("rate-limited ACCEPT %s", req.remote_addr)
                 return
